--
-- seasonManager
--
-- Versiongeschichte:
--
--	V0.1:
--	-Release 10.12.2016
--
--	V0.1.1
--	-Kleine Anpassungen
--	-Überschreiben des Wetter funktioniert jetzt zu jeder Uhrzeit
--	-Display zeigt aktuelle Jahreszeit
--	-Zip-Name geändert
--	-Release 11.12.2016
--
--	V0.2
--	-Config-XML eingebaut
--	-Release 17.12.2016
--
--	V0.2b
--	-Abfrage der Downloadquelle/Overlay deaktiviert
--	-Frühlingswetter verändert (von 'fog' zu 'cloudy')
--	-Laden der Config-XML verbessert 
--	-Release 18.12.2016
--
--	V0.3
--	-Dauer einer Jahreszeit kann nun verändert werden
--	-Display zeigt nun auch die zeitliche Position innerhalb einer Jahreszeit an
--	-Wetter-Funktion von Grund auf neu geschrieben
--	-Script enorm verkürzt
--	-Icon überarbeitet
--	-Release 23.12.2016
--
--	V0.3.1
--	-Wetter-Funktion generiert Wetter nun ab dem folgenden Tag
--	-Release 28.12.2016
--	
--	V0.4
--	-angepasste Beleuchtung
--	-Release 05.01.2017

seasonManager = {};
addModEventListener(seasonManager);
--function seasonManager.prerequisitesPresent(specializations)
    --return true;
--end;

function seasonManager:loadMap(name)

	g_currentMission.environment:addDayChangeListener(seasonManager);
	
	scriptVersion = "V0.4"
	
	print("--- seasonManager " .. scriptVersion .. " loaded --- (by Lefthandright)");
	
	seasonManager.loadConfigFile();
	seasonManager.seasonCheck();
	seasonManager.nextSeasonCheck();
	seasonManager.generateLights();
	
end;

function seasonManager:deleteMap()
	
end;

function seasonManager:loadConfigFile()

	path = getUserProfileAppPath();
	
	savegameIndex = g_currentMission.missionInfo.savegameIndex;
	savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
	end;
	
	--local configXML;
	configFile = savegameFolderPath .. "/seasonManager.xml";
	
	
	if fileExists(configFile) then
	
		print("--- seasonManager: loading " .. configFile .. " ---");
		configXML = loadXMLFile("seasonManager_XML", configFile, "seasonManager");
		version = getXMLString(configXML, "seasonManager.version");
		--source = getXMLString(configXML, "seasonManager.source");
		seasonDuration = getXMLInt(configXML, "seasonManager.seasonDuration");
		language = getXMLString(configXML, "seasonManager.language");
		displayX = getXMLFloat(configXML, "seasonManager.displayX");
		displayY = getXMLFloat(configXML, "seasonManager.displayY");
	
		saveXMLFile(configXML);
	
	else
		seasonManager.createConfigFile();
	end;
	
	--if source == "Enter the name of the mod portal where you downloaded the seasonManager in the XML-File" then
		--sourceID = 1;
	--elseif source == "Modhoster" then
		--sourceID = 2;
	--else
		--sourceID = 0;
	--end;

	if version ~= scriptVersion then
		seasonManager.updateConfigFile();
	end;
	
end;

function seasonManager.createConfigFile()

	print("--- seasonManager: creating " .. configFile .. " ---");
	--configXML = createXMLFile("seasonManager_XML", configFile, "seasonManager");
	--setXMLString(configXML, "seasonManager.version", scriptVersion);
	--setXMLString(configXML, "seasonManager.source", "Enter the name of the mod portal where you downloaded the seasonManager in the XML-File");
	--setXMLInt(configXML, "seasonManager.seasonDuration", 1);
	--setXMLString(configXML, "seasonManager.language", "Enter 'de' for german. Enter anything else for english");
	--setXMLFloat(configXML, "seasonManager.displayX", 0.75);
	--setXMLFloat(configXML, "seasonManager.displayY", 0.982);

	--saveXMLFile(configXML);
	
	version = scriptVersion;
	--source = "Enter the name of the mod portal where you downloaded the seasonManager in the XML-File";
	seasonDuration = 1;
	language = "Enter 'de' for german. Enter anything else for english";
	displayX = 0.730;
	displayY = 0.982;
	
end;

function seasonManager.updateConfigFile()
	print ("--- seasonManager: updating " .. configFile .. " ---");
	--delete(configXML);
	--seasonManager.createConfigFile();
	version = scriptVersion;
	setXMLString(configXML, "seasonManager.version", scriptVersion);
	saveXMLFile(configXML);
end;

function seasonManager:seasonCheck()
	
	local day = g_currentMission.environment.currentDay;
	
	currentSeason = (day / seasonDuration) % 4;
	
	if currentSeason > 0 and currentSeason <= 1 then
		currentSeasonID = "Summer";
	elseif currentSeason > 1 and currentSeason <= 2 then
		currentSeasonID = "Autumn";
	elseif currentSeason > 2 and currentSeason <= 3 then
		currentSeasonID = "Winter";
	elseif currentSeason == 0 or currentSeason <= 4 then
		currentSeasonID = "Spring";
	end;
	
	print ("--- seasonManager: " .. currentSeasonID .. " ---");
	
end;

function seasonManager:nextSeasonCheck()

	local day = g_currentMission.environment.currentDay;
	
	for counter = 0, seasonDuration do
	
		if ((day + counter) / seasonDuration) % 4 == 0 or ((day + counter) / seasonDuration) % 4 == 1 or ((day + counter) / seasonDuration) % 4 == 2 or ((day + counter) / seasonDuration) % 4 == 3 then
		
			daysToNextSeason = counter + 1;
			seasonPosition = seasonDuration - counter;
			print ("--- seasonManager: next season in " .. daysToNextSeason .. " day(s) ---");
			break
		
		else
			print ("--- seasonManager: searching next season ---");
		end;
	
	end;
	
end;

function seasonManager:save()
	
	seasonManager.createWeather(); 

	--local configXML;
	--local configFile = savegameFolderPath .. "/seasonManager.xml";
	
	print ("--- seasonManager: saving config to " .. configFile .. " ---");
	
	configXML = createXMLFile("seasonManager_XML", configFile, "seasonManager");
	setXMLString(configXML, "seasonManager.version", version);
	--setXMLString(configXML, "seasonManager.source", source);
	setXMLInt(configXML, "seasonManager.seasonDuration", seasonDuration);
	setXMLString(configXML, "seasonManager.language", language);
	setXMLFloat(configXML, "seasonManager.displayX", displayX);
	setXMLFloat(configXML, "seasonManager.displayY", displayY);
	
	saveXMLFile(configXML);
	
end;

function seasonManager:writeWeather()

	local key = "careerSavegame.environment.rains.rain(" .. weatherIndex .. ")";
	setXMLBool(savegameXML, key .. "#isValid", true);
	setXMLString(savegameXML, key .. "#rainTypeId", eventType);
	setXMLFloat(savegameXML, key .. "#duration", 720);
	setXMLFloat(savegameXML, key .. "#startDayTime", 540);
	setXMLInt(savegameXML, key .. "#startDay", eventDay);
	
	weatherIndex = weatherIndex + 1;
	eventDay = eventDay + 1;
	
	saveXMLFile(savegameXML);

end;

function seasonManager:createWeather()

	local day = g_currentMission.environment.currentDay;
	local hour = g_currentMission.environment.currentHour;
	savegameFolderPath = getUserProfileAppPath().."savegame"..g_currentMission.missionInfo.savegameIndex;
	savegameXML = loadXMLFile("careerSavegame", savegameFolderPath .. "/careerSavegame.xml");

	seasonManager.seasonCheck();
	seasonManager.nextSeasonCheck();
	
	print ("--- seasonManager: creating weather ---");
	
	if currentSeasonID == "Spring" then
		
		weatherIndex = 0;
		eventDay = day + 1;
		
		if daysToNextSeason > 1 then
			daysToNextSeasonAlternate = daysToNextSeason - 1;
			for counter = 1, daysToNextSeasonAlternate do
				eventType = "cloudy";
				seasonManager.writeWeather();
			end;
		end;
		
		eventDay = day + daysToNextSeason + seasonDuration - 1;
		
		for counter = 1, seasonDuration do
			eventType = "rain";
			seasonManager.writeWeather();
		end;
		
		for counter = 1, seasonDuration do
			eventType = "hail";
			seasonManager.writeWeather();
		end;
		
		for counter = 1, seasonDuration do
			eventType = "cloudy";
			seasonManager.writeWeather();
		end;
	
	elseif currentSeasonID == "Summer" then
		
		weatherIndex = 0;
		eventDay = day + daysToNextSeason - 1;
		for counter = 1, seasonDuration do
			eventType = "rain";
			seasonManager.writeWeather();
		end;
		
		for counter = 1, seasonDuration do
			eventType = "hail";
			seasonManager.writeWeather();
		end;
		
		for counter = 1, seasonDuration do
			eventType = "cloudy";
			seasonManager.writeWeather();
		end;
		
	elseif currentSeasonID == "Autumn" then
		
		weatherIndex = 0;
		eventDay = day + 1;
		
		if daysToNextSeason > 1 then
			daysToNextSeasonAlternate = daysToNextSeason - 1;
			for counter = 1, daysToNextSeasonAlternate do
				eventType = "rain";
				seasonManager.writeWeather();
			end;
		end
		
		eventDay = day + daysToNextSeason - 1;
		
		for counter = 1, seasonDuration do
			eventType = "hail";
			seasonManager.writeWeather();
		end;
		
		for counter = 1, seasonDuration do
			eventType = "cloudy";
			seasonManager.writeWeather();
		end;
		
		eventDay = eventDay + seasonDuration;
		
		for counter = 1, seasonDuration do
			eventType = "rain";
			seasonManager.writeWeather();
		end;
	
	elseif currentSeasonID == "Winter" then
		
		weatherIndex = 0;
		eventDay = day + 1;
		
		if daysToNextSeason > 1 then
			daysToNextSeasonAlternate = daysToNextSeason - 1;
			for counter = 1, daysToNextSeasonAlternate do
				eventType = "hail";
				seasonManager.writeWeather();
			end;
		end;
		
		eventDay = day + daysToNextSeason - 1;
		
		for counter = 1, seasonDuration do
			eventType = "cloudy";
			seasonManager.writeWeather();
		end;
		
		eventDay = eventDay + seasonDuration;
		
		for counter = 1, seasonDuration do
			eventType = "rain";
			seasonManager.writeWeather();
		end;
		
		for counter = 1, seasonDuration do
			eventType = "hail";
			seasonManager.writeWeather();
		end;
		
	end;
	
end;

function seasonManager:generateLights()
	
	local cme = g_currentMission.environment;
	
	if currentSeasonID == "Summer" then
	
		cme.ambientCurve = AnimCurve:new(linearInterpolator3);
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 0*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 3.75*60});
		cme.ambientCurve:addKeyframe({x=0.065, y=0.065, z=0.075, time = 4.01*60});
		cme.ambientCurve:addKeyframe({x=0.065, y=0.065, z=0.075, time = 4.02*60});
		cme.ambientCurve:addKeyframe({x=0.040, y=0.040, z=0.060, time = 4.25*60});
		cme.ambientCurve:addKeyframe({x=0.080, y=0.080, z=0.120, time = 5*60});
		cme.ambientCurve:addKeyframe({x=0.220, y=0.140, z=0.120, time = 6*60});
		cme.ambientCurve:addKeyframe({x=0.500, y=0.300, z=0.150, time = 9*60});
		cme.ambientCurve:addKeyframe({x=0.500, y=0.300, z=0.150, time = 17*60});
		cme.ambientCurve:addKeyframe({x=0.220, y=0.140, z=0.120, time = 18*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 21*60});
		cme.ambientCurve:addKeyframe({x=0.024, y=0.024, z=0.036, time = 21.85*60});
		cme.ambientCurve:addKeyframe({x=0.072, y=0.072, z=0.088, time = 22.01*60});
		cme.ambientCurve:addKeyframe({x=0.072, y=0.072, z=0.088, time = 22.02*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 22.15*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 24*60});
		
	elseif currentSeasonID == "Autumn" then
	
		cme.ambientCurve = AnimCurve:new(linearInterpolator3);
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 0*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 3.75*60});
		cme.ambientCurve:addKeyframe({x=0.065, y=0.065, z=0.075, time = 4.01*60});
		cme.ambientCurve:addKeyframe({x=0.065, y=0.065, z=0.075, time = 4.02*60});
		cme.ambientCurve:addKeyframe({x=0.040, y=0.040, z=0.060, time = 4.25*60});
		cme.ambientCurve:addKeyframe({x=0.080, y=0.080, z=0.140, time = 5*60});
		cme.ambientCurve:addKeyframe({x=0.140, y=0.140, z=0.180, time = 6*60});
		cme.ambientCurve:addKeyframe({x=0.180, y=0.180, z=0.220, time = 9*60});
		cme.ambientCurve:addKeyframe({x=0.180, y=0.180, z=0.220, time = 17*60});
		cme.ambientCurve:addKeyframe({x=0.160, y=0.140, z=0.160, time = 18*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 21*60});
		cme.ambientCurve:addKeyframe({x=0.024, y=0.024, z=0.036, time = 21.85*60});
		cme.ambientCurve:addKeyframe({x=0.072, y=0.072, z=0.088, time = 22.01*60});
		cme.ambientCurve:addKeyframe({x=0.072, y=0.072, z=0.088, time = 22.02*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 22.15*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 24*60});
		
	elseif currentSeasonID == "Winter" then
	
		cme.ambientCurve = AnimCurve:new(linearInterpolator3);
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 0*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 3.75*60});
		cme.ambientCurve:addKeyframe({x=0.065, y=0.065, z=0.075, time = 4.01*60});
		cme.ambientCurve:addKeyframe({x=0.065, y=0.065, z=0.075, time = 4.02*60});
		cme.ambientCurve:addKeyframe({x=0.040, y=0.040, z=0.060, time = 4.25*60});
		cme.ambientCurve:addKeyframe({x=0.040, y=0.060, z=0.140, time = 5*60});
		cme.ambientCurve:addKeyframe({x=0.060, y=0.080, z=0.160, time = 6*60});
		cme.ambientCurve:addKeyframe({x=0.080, y=0.100, z=0.200, time = 9*60});
		cme.ambientCurve:addKeyframe({x=0.080, y=0.100, z=0.200, time = 17*60});
		cme.ambientCurve:addKeyframe({x=0.100, y=0.080, z=0.180, time = 18*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 21*60});
		cme.ambientCurve:addKeyframe({x=0.024, y=0.024, z=0.036, time = 21.85*60});
		cme.ambientCurve:addKeyframe({x=0.072, y=0.072, z=0.088, time = 22.01*60});
		cme.ambientCurve:addKeyframe({x=0.072, y=0.072, z=0.088, time = 22.02*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 22.15*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 24*60});
		
	elseif currentSeasonID == "Spring" then
	
		cme.ambientCurve = AnimCurve:new(linearInterpolator3);
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 0*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 3.75*60});
		cme.ambientCurve:addKeyframe({x=0.065, y=0.065, z=0.075, time = 4.01*60});
		cme.ambientCurve:addKeyframe({x=0.065, y=0.065, z=0.075, time = 4.02*60});
		cme.ambientCurve:addKeyframe({x=0.040, y=0.040, z=0.060, time = 4.25*60});
		cme.ambientCurve:addKeyframe({x=0.080, y=0.080, z=0.140, time = 5*60});
		cme.ambientCurve:addKeyframe({x=0.140, y=0.140, z=0.120, time = 6*60});
		cme.ambientCurve:addKeyframe({x=0.220, y=0.180, z=0.180, time = 9*60});
		cme.ambientCurve:addKeyframe({x=0.220, y=0.180, z=0.180, time = 17*60});
		cme.ambientCurve:addKeyframe({x=0.160, y=0.140, z=0.160, time = 18*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 21*60});
		cme.ambientCurve:addKeyframe({x=0.024, y=0.024, z=0.036, time = 21.85*60});
		cme.ambientCurve:addKeyframe({x=0.072, y=0.072, z=0.088, time = 22.01*60});
		cme.ambientCurve:addKeyframe({x=0.072, y=0.072, z=0.088, time = 22.02*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 22.15*60});
		cme.ambientCurve:addKeyframe({x=0.020, y=0.020, z=0.032, time = 24*60});
		
	end;
	
end;

function seasonManager:dayChanged()

	seasonManager.seasonCheck();
	seasonManager.nextSeasonCheck();
	seasonManager.generateLights();
	print ("--- seasonManager: day " .. tostring(g_currentMission.environment.currentDay) .. " ---");
	
end;

function seasonManager:mouseEvent(posX, posY, isDown, isUp, button)

end;

function seasonManager:keyEvent(unicode, sym, Modifier, isDown)

end;

function seasonManager:update()
	
end;

function seasonManager:draw()
	
	setTextColor(1, 1, 1, 1);
	
	if language == "de" then
		if currentSeasonID == "Summer" then
			renderText(displayX, displayY, 0.02, seasonPosition .. "/" .. seasonDuration .. " Sommer");
		elseif currentSeasonID == "Autumn" then
			renderText(displayX, displayY, 0.02, seasonPosition .. "/" .. seasonDuration .. " Herbst");
		elseif currentSeasonID == "Winter" then
			renderText(displayX, displayY, 0.02, seasonPosition .. "/" .. seasonDuration .. " Winter");
		elseif currentSeasonID == "Spring" then
			renderText(displayX, displayY, 0.02, seasonPosition .. "/" .. seasonDuration .. " Frühling");
		end;
	else
		renderText(displayX, displayY, 0.02, seasonPosition .. "/" .. seasonDuration .. " " .. currentSeasonID);
	end;
	
	--if sourceID == 1 then
		--if language == "de" then
			--renderText(0.002,0.982,0.02, "Gibt den Namen des Modportals, wo du den seasonManager heruntergeladen hast, in die XML-Datei ein");
		--else
			--renderText(0.002,0.982,0.02, "Enter the name of the mod portal where you downloaded the seasonManager in the XML-File");
		--end;
	--end;
	
	--if sourceID == 0 then
		--if language == "de" then
			--renderText(0.002,0.982,0.02, "Lade den seasonManager auf Modhoster herunter, um den Entwickler zu unterstützen");
		--else
			--renderText(0.002,0.982,0.02, "Download the seasonManager on Modhoster to support the developer");
		--end;
	--end;
	
	setTextColor(1, 1, 1, 0);
	
end;

g_careerScreen.saveSavegame = Utils.appendedFunction(g_careerScreen.saveSavegame, seasonManager.save);